# -*- coding: utf-8 -*-
"""
Created on Sun Oct  9 21:18:13 2022

@author: Jinyi Zhang
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from copy import deepcopy, copy
from qstock.data.trade import get_data

def calcTR(high, low, close):
  '''Calculate True Range'''
  return np.max(np.abs([high-low, close-low, low-close]))

def getStratStats(log_returns: pd.Series,
  risk_free_rate: float = 0.02):
  stats = {}  # Total Returns
  stats['tot_returns'] = np.exp(log_returns.sum()) - 1  
  
  # Mean Annual Returns
  stats['annual_returns'] = np.exp(log_returns.mean() * 252) - 1  
  
  # Annual Volatility
  stats['annual_volatility'] = log_returns.std() * np.sqrt(252)
  
  # Sortino Ratio
  annualized_downside = log_returns.loc[log_returns<0].std() * \
    np.sqrt(252)
  stats['sortino_ratio'] = (stats['annual_returns'] - \
    risk_free_rate) / annualized_downside  
  
  # Sharpe Ratio
  stats['sharpe_ratio'] = (stats['annual_returns'] - \
    risk_free_rate) / stats['annual_volatility']  
  
  # Max Drawdown
  cum_returns = log_returns.cumsum() - 1
  peak = cum_returns.cummax()
  drawdown = peak - cum_returns
  max_idx = drawdown.argmax()
  stats['max_drawdown'] = 1 - np.exp(cum_returns[max_idx]) \
    / np.exp(peak[max_idx])
  
  # Max Drawdown Duration
  strat_dd = drawdown[drawdown==0]
  strat_dd_diff = strat_dd.index[1:] - strat_dd.index[:-1]
  strat_dd_days = strat_dd_diff.map(lambda x: x.days).values
  strat_dd_days = np.hstack([strat_dd_days,
    (drawdown.index[-1] - strat_dd.index[-1]).days])
  stats['max_drawdown_duration'] = strat_dd_days.max()
  return {k: np.round(v, 4) if type(v) == np.float_ else v
          for k, v in stats.items()}

class TurtleSystem:

  def __init__(self, codes, init_account_size=10000, risk_level=2, r_max=0.02,
               sys1_entry=20, sys1_exit=10, sys2_entry=55, sys2_exit=20,
               atr_periods=20, sys1_allocation=0.5, risk_reduction_rate=0.1,
               risk_reduction_level=0.2, unit_limit=5, pyramid_units=1, 
               start='2000-01-01', end='2020-12-31', shorts=True):
    '''
    :codes:股票代码或简称
    :init_account_size: int that sets initial trading capital
    :risk_level: float used to determine the stop loss distance by multiplying
      this value with N.
    :r_max: float max percentage of account that a trade can risk.
    :sys1_entry: int determines number of breakout days for System 1 to generate
      a buy signal.
    :sys1_exit: int determines number of breakout days for System 1 to generate
      a sell signal.
    :sys2_entry: int determines number of breakout days for System 2 to generate
      a buy signal.
    :sys2_exit: int determines number of breakout days for System 2 to generate
      a sell signal.
    :sys1_allocation: float to balance capital allocation 
      between System 1 and 2.
    :start: str first date for getting data.
    :end: str end date for getting data.
    :shorts: bool to allow short positions if True.
    :atr_periods: int number of days used to calculate SMA of N.
    :risk_reduction_rate: float < 1 represents the amount of loss the system
      sees before it reduces its trading size.
    :risk_reduction_level: float < 1 represents each increment in risk the
      the system reduces as it loses capital below its initial size.
    '''
    if isinstance(codes,str):
        codes=[codes]
    self.codes = codes
    self.init_account_size = init_account_size
    self.cash = init_account_size
    self.portfolio_value = init_account_size
    self.risk_level = risk_level
    self.r_max = r_max
    self.sys1_entry = sys1_entry
    self.sys1_exit = sys1_exit
    self.sys2_entry = sys2_entry
    self.sys2_exit = sys2_exit
    self.sys1_allocation = sys1_allocation
    self.sys2_allocation = 1 - sys1_allocation
    self.start = start
    self.end = end
    self.atr_periods = atr_periods
    self.shorts = shorts
    self.last_s1_win = {t: False for t in self.codes}
    self.unit_limit = unit_limit
    self.risk_reduction_level = risk_reduction_level
    self.risk_reduction_rate = risk_reduction_rate
    self.pyramid_units = pyramid_units
    self.sys_list = ['S1', 'S2']

    self._prep_data()

  def _prep_data(self):
    self.data = self._get_data()
    self._calc_breakouts()
    self._calc_N()

  def _get_data(self):
    # Gets data for all codes
    df = get_data(self.codes,start=self.start,end=self.end,fqt=2)
    df.set_index([df.index,'name'],inplace=True)
    df=df.unstack()
    df.ffill(inplace=True)
    df=df.swaplevel(axis=1)
    return df

  def _calc_breakouts(self):
    # Gets breakouts for all codes
    for t in self.codes:
      # Breakouts for enter long position (EL), exit long (ExL)
      # enter short (ES), exit short (ExS)
      self.data[t, 'S1_EL'] = self.data[t]['close'].rolling(
          self.sys1_entry).max()
      self.data[t, 'S1_ExL'] = self.data[t]['close'].rolling(
          self.sys1_exit).min()
      self.data[t, 'S2_EL'] = self.data[t]['close'].rolling(
          self.sys2_entry).max()
      self.data[t, 'S2_ExL'] = self.data[t]['close'].rolling(
          self.sys2_exit).min()

      if self.shorts:
        self.data[t, 'S1_ES'] = self.data[t]['close'].rolling(
            self.sys1_entry).min()
        self.data[t, 'S1_ExS'] = self.data[t]['close'].rolling(
            self.sys1_exit).max()
        self.data[t, 'S2_ES'] = self.data[t]['close'].rolling(
            self.sys2_entry).min()
        self.data[t, 'S2_ExS'] = self.data[t]['close'].rolling(
            self.sys2_exit).max()

  def _calc_N(self):
    # Calculates N for all codes
    for t in self.codes:
      tr = self.data[t].apply(
          lambda x: calcTR(x['high'], x['low'], x['close']), axis=1)
      self.data[t, 'N'] = tr.rolling(self.atr_periods).mean()

  def _check_cash_balance(self, shares, price):
    # Checks to see if we have enough cash to make purchase.
    # If not, resizes position to lowest feasible level
    if self.cash <= shares * price:
      shares = np.floor(self.cash / price)
    return shares

  def _adjust_risk_units(self, units):
    # Scales units down by 20% for every 10% of capital that has been lost
    # under default settings.
    cap_loss = 1 - self.portfolio_value / self.init_account_size
    if cap_loss > self.risk_reduction_level:
      scale = np.floor(cap_loss / self.risk_reduction_level)
      units *= (1 - scale * self.risk_reduction_rate)
    return units

  def _calc_portfolio_value(self, portfolio):
    pv = sum([v1['value'] for v0 in portfolio.values() if type(v0) is dict 
              for k1, v1 in v0.items() if v1 is not None])
    pv += self.cash
    if np.isnan(pv):
      raise ValueError(f"PV = {pv}\n{portfolio}")
    return pv

  def _get_units(self, system):
    sys_all = self.sys1_allocation if system == 1 else self.sys2_allocation
    dollar_units = self.r_max * self.portfolio_value * sys_all
    dollar_units = self._adjust_risk_units(dollar_units)
    return dollar_units

  def _size_position(self, data, dollar_units):
    shares = np.floor(dollar_units / (
        self.risk_level * data['N'] * data['close']))
    return shares

  def _run_system(self, ticker, data, position, system=1):
    S = system # System number
    price = data['close']
    if np.isnan(price):
      # Return current position in case of missing data
      return position
    N = data['N']
    dollar_units = self._get_units(S)
    shares =  0
    if position is None:
      if price == data[f'S{S}_EL']: # Buy on breakout
        if S == 1 and self.last_s1_win[ticker]:
          self.last_s1_win[ticker] = False
          return None
        shares = self._size_position(data, dollar_units)
        stop_price = price - self.risk_level * N
        long = True
      elif self.shorts:
        if price == data[f'S{S}_ES']: # Sell short
          if S == 1 and self.last_s1_win[ticker]:
            self.last_s1_win[ticker] = False
            return None
          shares = self._size_position(data, dollar_units)
          stop_price = price + self.risk_level * N
          long = False
      else:
        return None
      if shares == 0:
        return None
      # Ensure we have enough cash to trade
      shares = self._check_cash_balance(shares, price)
      value = price * shares

      self.cash -= value
      position = {'units': 1,
                  'shares': shares,
                  'entry_price': price,
                  'stop_price': stop_price,
                  'entry_N': N,
                  'value': value,
                  'long': long}
      if np.isnan(self.cash) or self.cash < 0:
        raise ValueError(f"Cash Error\n{S}-{ticker}\n{data}\n{position}")

    else:
      if position['long']:
        # Check to exit existing long position
        if price == data[f'S{S}_ExL'] or price <= position['stop_price']:
          self.cash += position['shares'] * price
          if price >= position['entry_price']:
            self.last_s1_win[ticker] = True
          else:
            self.last_s1_win[ticker] = False
          position = None
        # Check to pyramid existing position
        elif position['units'] < self.unit_limit:
          if price >= position['entry_price'] + position['entry_N']:
            shares = self._size_position(data, dollar_units)
            shares = self._check_cash_balance(shares, price)
            self.cash -= shares * price
            stop_price = price - self.risk_level * N
            avg_price = (position['entry_price'] * position['shares'] +
                         shares * price) / (position['shares'] + shares)
            position['entry_price'] = avg_price
            position['shares'] += shares
            position['stop_price'] = stop_price
            position['units'] += 1
      else:
        # Check to exit existing short position
        if price == data[f'S{S}_ExS'] or price >= position['stop_price']:
          self.cash += position['shares'] * price
          if S == 1:
            if price <= position['entry_price']:
              self.last_s1_win[ticker] = True
            else:
              self.last_s1_win[ticker] = False
          position = None
        # Check to pyramid existing position
        elif position['units'] < self.unit_limit:
          if price <= position['entry_price'] - position['entry_N']:
            shares = self._size_position(data, dollar_units)
            shares = self._check_cash_balance(shares, price)
            self.cash -= shares * price
            stop_price = price + self.risk_level * N
            avg_price = (position['entry_price'] * position['shares'] +
                         shares * price) / (position['shares'] + shares)
            position['entry_price'] = avg_price
            position['shares'] += shares
            position['stop_price'] = stop_price
            position['units'] += 1

      if position is not None:
        # Update value at each time step
        position['value'] = position['shares'] * price
        
    return position

  def run(self):
    # Runs backtest on the turtle strategy
    self.portfolio = {}
    position = {s: 
                  {t: None for t in self.codes}
                for s in self.sys_list}

    for i, (ts, row) in enumerate(self.data.iterrows()):
      for t in self.codes:
        for s, system in enumerate(self.sys_list):
          position[system][t] = self._run_system(t, row[t], position[system][t])
      self.portfolio[i] = deepcopy(position)
      self.portfolio[i]['date'] = ts
      self.portfolio[i]['cash'] = copy(self.cash)
      self.portfolio_value = self._calc_portfolio_value(self.portfolio[i])

  def get_portfolio_values(self):
    vals = []
    for v in self.portfolio.values():
      pv = sum([v1['value'] for v0 in v.values() if type(v0) is dict 
              for k1, v1 in v0.items() if v1 is not None])
      pv += v['cash']
      vals.append(pv)
    return pd.Series(vals, index=self.data.index)

  def get_system_data_dict(self):
    sys_dict = {}
    cols = ['units', 'shares', 'entry_price', 'stop_price',
      'entry_N', 'value', 'long']
    X = np.empty(shape=(len(cols))) 
    X[:] = np.nan
    index = [v['date'] for v in self.portfolio.values()]
    for s in self.sys_list:
      for t in self.codes:
        df = pd.DataFrame()
        for i, v in enumerate(self.portfolio.values()):
          d = v[s][t]
          if d is None:
            if i == 0:
              _array = X.copy()
            else:
              _array = np.vstack([_array, X])
          else:
            vals = np.array([float(d[i]) for i in cols])
            if i == 0:
              _array = vals.copy()
            else:
              _array = np.vstack([_array, vals])
        df = pd.DataFrame(_array, columns=cols, index=index)
        sys_dict[(s, t)] = df.copy()
    return sys_dict

  def get_transactions(self):
    ddict = self.get_system_data_dict()
    transactions = pd.DataFrame()
    for k, v in ddict.items():
      df = pd.concat([v, self.data[k[1]].copy()], axis=1)
      df.fillna(0, inplace=True)
      rets = df['close'] / df['entry_price'].shift(1) -1
      trans = pd.DataFrame(rets[df['shares'].diff()<0], 
        columns=['Returns'])
      trans['System'] = k[0]
      trans['Ticker'] = k[1]
      trans['Long'] = df['long'].shift(1).loc[df['shares'].diff()<0]
      trans['Units'] = df['units'].shift(1).loc[df['shares'].diff()<0]
      trans['Entry_Price'] = df['entry_price'].shift(1).loc[
        df['shares'].diff()<0]
      trans['Sell_Price'] = df['close'].loc[df['shares'].diff()<0]
      trans['Shares'] = df['shares'].shift(1).loc[df['shares'].diff()<0]
      trans.reset_index(inplace=True)
      trans.rename(columns={'index': 'Date'}, inplace=True)
      transactions = pd.concat([transactions, trans.copy()])

    transactions.reset_index(inplace=True)
    transactions.drop('index', axis=1, inplace=True)
    return transactions


def back_test(codes,index='sh',init_account_size=1000000.0, start='2010-01-01',end='2022-09-30'):
    sys = TurtleSystem(codes=codes, init_account_size=init_account_size, start=start,end=end)
    sys.run()
    port_values = sys.get_portfolio_values()
    returns = port_values / port_values.shift(1)
    log_returns = np.log(returns)
    cum_rets = log_returns.cumsum()
    base = get_data(index,start=sys.start, end=sys.end)
    base['returns'] = base['close'] / base['close'].shift(1)
    base['log_returns'] = np.log(base['returns'])
    base['cum_rets'] = base['log_returns'].cumsum()
    plt.figure(figsize=(15, 7))
    plt.plot((np.exp(cum_rets) -1 )* 100, label='海龟交易策略')
    plt.plot((np.exp(base['cum_rets']) - 1) * 100, label='基准指数')
    plt.xlabel('Date')
    plt.ylabel('Returns (%)')
    plt.title('Cumulative Portfolio Returns')
    plt.legend()
    plt.tight_layout()
    plt.show() 



# -*- coding: utf-8 -*-
"""
Created on Sun Aug  6 10:19:19 2023

@author: tkfy9
"""

import backtrader as bt
import qstock as qs
import warnings

# 忽略UserWarning（缺失zipline.assets ）
warnings.filterwarnings("ignore",  message="Module \"zipline.assets\"  not found")
import pyfolio as pf

def bt_result(code='sh',start='2000-01-01', end=None,
              strategy=None,startcash=10000000.0,commission=0.00):
    '''code:回测代码或简称列表（list）或数据的字典格式
       start，end为回测起始日期，格式为'2023-08-04'
       strategy:策略名称
       startcash：初始资金
       commission：手续费
    '''
    print('抱歉，该接口仅限知识星球会员使用，详情可添加个人微信sky2blue2，获取8折优惠')
    pass


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 30 22:05:59 2022

@author: Jinyi Zhang
"""
import re
import pandas as pd
import json
import requests
import time
import signal
import hashlib
from datetime import datetime,timedelta
from bs4 import BeautifulSoup

import multitasking
from func_timeout import func_set_timeout
signal.signal(signal.SIGINT, multitasking.killall)

######新闻资讯数据
def news_data(news_type=None,start=None,end=None,code=None):
    
    '''news_type:新闻类型：cctv'或'新闻联播'
    'js'或'金十数据'；'stock' 或'个股新闻'
    不输入参数，默认输出财联社电报新闻数据
    start:起始日期，如'20220930',不输入默认当前最新日期
    end:结束日期，如'20221001'，不输入默认当前最新日期
    stock：个股代码
    
    '''
        
    if news_type =='cctv' or news_type =='新闻联播':
        return news_cctv(start,end)
    elif news_type=='js' or news_type=='金十数据':
        return news_js(start,end)
            
    elif code is not None or news_type in ['stock','个股新闻','个股']:
        return stock_news(code)
    else:
        return news_cls()


def news_cls():
    """
    财联社-电报
    https://www.cls.cn/telegraph
    :return: 财联社-电报
    :rtype: pandas.DataFrame
    """

    current_time = int(time.time())
    url = "https://www.cls.cn/nodeapi/telegraphList"
    params = {
        "app": "CailianpressWeb",
        "category": "",
        "lastTime": current_time,
        "last_time": current_time,
        "os": "web",
        "refresh_type": "1",
        "rn": "2000",
        "sv": "7.7.5",
    }
    text = requests.get(url, params=params).url.split("?")[1]
    if not isinstance(text, bytes):
        text = bytes(text, "utf-8")
    sha1 = hashlib.sha1(text).hexdigest()
    code = hashlib.md5(sha1.encode()).hexdigest()
    
    params = {
        "app": "CailianpressWeb",
        "category": "",
        "lastTime": current_time,
        "last_time": current_time,
        "os": "web",
        "refresh_type": "1",
        "rn": "2000",
        "sv": "7.7.5",
        "sign": code,
    }
    headers = {
        "Accept": "application/json, text/plain, */*",
        "Accept-Encoding": "gzip, deflate, br",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Connection": "keep-alive",
        "Content-Type": "application/json;charset=utf-8",
        "Host": "www.cls.cn",
        "Pragma": "no-cache",
        "Referer": "https://www.cls.cn/telegraph",
        "sec-ch-ua": '".Not/A)Brand";v="99", "Google Chrome";v="103", "Chromium";v="103"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
        "Sec-Fetch-Dest": "empty",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Site": "same-origin",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
    }
    data = requests.get(url, headers=headers, params=params).json()
    df = pd.DataFrame(data["data"]["roll_data"])
    df = df[["title", "content", "ctime"]]
    df["ctime"] = pd.to_datetime(
        df["ctime"], unit="s", utc=True
    ).dt.tz_convert("Asia/Shanghai")
    df.columns = ["标题", "内容", "发布时间"]
    df.sort_values(["发布时间"], inplace=True)
    df.reset_index(inplace=True, drop=True)
    df["发布日期"] = df["发布时间"].dt.date
    df["发布时间"] = df["发布时间"].dt.time
    return df

def get_news_cctv(date= None):
    """
    新闻联播文字稿
    https://tv.cctv.com/lm/xwlb/?spm=C52056131267.P4y8I53JvSWE.0.0
    date: 需要获取数据的日期
    """
    
    now=datetime.now()
    now_date=now.strftime('%Y%m%d')
    if date is None:
        date=now_date
    else:
        date=''.join(date.split('-'))
    if  date>=now_date:
        date=(now-timedelta(1)).strftime('%Y%m%d')
       
    url = f"http://cctv.cntv.cn/lm/xinwenlianbo/{date}.shtml"
    res = requests.get(url)
    title_list = []
    content_list = []
    headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "Accept-Encoding": "gzip, deflate",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Cookie": "cna=DLYSGBDthG4CAbRVCNxSxGT6",
            "Host": "tv.cctv.com",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36",
    }
    if int(date) <= int("20130708"):
        res.encoding = "gbk"
        raw_list = re.findall(r"title_array_01\((.*)", res.text)
        page_url = [
            re.findall("(http.*)", item)[0].split("'")[0]
            for item in raw_list[1:]]   

    elif int(date) < int("20160203"):
        res.encoding = "utf-8"
        soup = BeautifulSoup(res.text, "lxml")
        page_url = [
            item.find("a")["href"]
            for item in soup.find(
                "div", attrs={"id": "contentELMT1368521805488378"}
            ).find_all("li")[1:]]

    else:
        url = f"https://tv.cctv.com/lm/xwlb/day/{date}.shtml"
        res = requests.get(url)
        res.encoding = "utf-8"
        soup = BeautifulSoup(res.text, "lxml")
        page_url = [item.find("a")["href"] for item in soup.find_all("li")[1:]]
        
    for page in page_url:
        try:
            r = requests.get(page, headers=headers)
            r.encoding = "utf-8"
            soup = BeautifulSoup(r.text, "lxml")
            if soup.find("h3"):
                title = soup.find("h3").text
            else:
                title = soup.find("div", attrs={"class": "tit"}).text
            if soup.find("div", attrs={"class": "cnt_bd"}):
                content = soup.find("div", attrs={"class": "cnt_bd"}).text
            else:
                content = soup.find(
                    "div", attrs={"class": "content_area"}
                    ).text
            title_list.append(
                    title.strip("[视频]").strip().replace("\n", " ")
                )
            content_list.append(
                    content.strip()
                    .strip("央视网消息(新闻联播)：")
                    .strip("央视网消息（新闻联播）：")
                    .strip("(新闻联播)：")
                    .strip()
                    .replace("\n", " ")
                )
        except:
            continue
    df = pd.DataFrame(
            [[date] * len(title_list), title_list, content_list],
            index=["date", "title", "content"],
        ).T
    return df

def get_dates(start=None,end=None):
    if start is None:
        start=(datetime.now()).strftime('%Y%m%d')
    if end is None:
        end=(datetime.now()).strftime('%Y%m%d')
    if start>end:
        start=end
    dates=pd.date_range(start,end)
    dates=[s.strftime('%Y%m%d') for s in dates]
    return dates

def news_cctv(start=None,end=None):
    '''获取某日期期间的所有新闻联播数据
    start:起始日期，如'20220930'
    end:结束日期，如'20221001'
    '''
    dates=get_dates(start,end)
    data_list=[]
    @multitasking.task
    @func_set_timeout(5)
    def run(date):
        data = get_news_cctv(date)
        data_list.append(data)
       
    for date in dates:
        try:
            run(date)
        except:
            continue
    multitasking.wait_for_tasks()
    # 转换为dataframe
    df = pd.concat(data_list, axis=0,ignore_index=True)
    return df

def news_js(start,end):
    '''获取某日期期间的所有金十数据-市场快讯数据
    start:起始日期，如'20220930'
    end:结束日期，如'20221001'
    '''
    dates=get_dates(start,end)
    data_list=[]
    @multitasking.task
    @func_set_timeout(5)
    def run(date):
        data = get_js_news(date)
        data_list.append(data)
       
    for date in dates:
        run(date)
    multitasking.wait_for_tasks()
    # 转换为dataframe
    df = pd.concat(data_list, axis=0,ignore_index=True)
    return df


def get_js_news(date=None):
    """
    金十数据-市场快讯
    https://www.jin10.com/
    date: 日期
    """
    if date is None:
        date=(datetime.now()+timedelta(1)).strftime('%Y%m%d')
    else:
        date=''.join(date.split('-'))
        date=datetime.strptime(date,'%Y%m%d')+timedelta(1)
        date=date.strftime('%Y%m%d')
    
    url = "https://flash-api.jin10.com/get_flash_list"
    params = {
        "channel": "-8200",
        "vip": "1",
        "t": "1625623640730",
        "max_time": date,
    }
    headers = {
        "accept": "application/json, text/plain, */*",
        "accept-encoding": "gzip, deflate, br",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
        "cache-control": "no-cache",
        "handleerror": "true",
        "origin": "https://www.jin10.com",
        "pragma": "no-cache",
        "referer": "https://www.jin10.com/",
        "sec-ch-ua": '" Not;A Brand";v="99", "Google Chrome";v="91", "Chromium";v="91"',
        "sec-ch-ua-mobile": "?0",
        "sec-fetch-dest": "empty",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "same-site",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36",
        "x-app-id": "bVBF4FyRTn5NJF5n",
        "x-version": "1.0.0",
    }
    data = requests.get(url, params=params, headers=headers).json()
    temp_df = pd.DataFrame(data["data"])
    temp_list = []
    for item in temp_df["data"]:
        if "content" in item.keys():
            temp_list.append(item["content"])
        elif "pic" in item.keys():
            temp_list.append(item["pic"])
        else:
            temp_list.append("-")
    df = pd.DataFrame([temp_df["time"].to_list(), temp_list]).T
    df.columns = ["datetime", "content"]
    df.sort_values(["datetime"], inplace=True)
    df.reset_index(inplace=True, drop=True)
    return df

def stock_news(stock):
    """
    东方财富-个股新闻 
    stock: 股票代码 
    """
    url = "https://search-api-web.eastmoney.com/search/jsonp" 
    
    # 动态生成 callback 参数 
    timestamp = int(time.time()  * 1000)
    cb = f"jQuery3510875346244069884_{timestamp}"
    
    # 构造 param 参数 
    search_scope = "default"
    sort = "default"
    page_index = 1 
    page_size = 100 
    pre_tag = "<em>"
    post_tag = "</em>"
    
    # 新的 param 构造方式（假设 API 结构发生变化）
    new_param = {
        "uid": "",
        "keyword": stock,
        "type": ["cmsArticle"],
        "client": "web",
        "clientType": "web",
        "clientVersion": "curr",
        "param": {
            "cmsArticle": {
                "searchScope": search_scope,
                "sort": sort,
                "pageIndex": page_index,
                "pageSize": page_size,
                "preTag": pre_tag,
                "postTag": post_tag 
            }
        }
    }
    
    params = {
        "cb": cb,
        "param": json.dumps(new_param), 
        "_": timestamp + 1,
    }
    
    try:
        res = requests.get(url,  params=params)
        res.raise_for_status()   # 检查 HTTP 请求是否成功 
        
        data_text = res.text  
        
        # 解析 JSON 数据 
        data_json = json.loads(data_text.strip(f"{cb}(")[:-1]) 
        
        # 检查数据结构 
        if "result" not in data_json:
            print("No result found in the response.")
            return None 
            
        result = data_json["result"]
        
        # 检查 cmsArticle 是否存在（假设新键为 cmsArticle）
        if "cmsArticle" not in result:
            print("The key 'cmsArticle' is missing in the result.")
            return None 
            
        df = pd.DataFrame(result["cmsArticle"])
        
        # 清洗标题和内容中的 HTML 标签 
        df['title'] = df['title'].apply(lambda s: re.sub(r'[<em></em>]',  '', s))
        df['content'] = df['content'].apply(lambda s: re.sub(r'[<em></em>]',  '', s))
        
        df['code'] = stock 
        df = df[['date', 'code', 'title', 'content', 'url']]
        
        return df 
        
    except requests.exceptions.RequestException  as e:
        print(f"Request failed: {e}")
        return None 
    except json.JSONDecodeError:
        print("Failed to decode JSON response.")
        return None 
    except KeyError as e:
        print(f"Missing key in JSON response: {e}")
        return None 


# -*- coding: utf-8 -*-
"""
Created on Mon Nov 11 17:39:06 2019

@author: Jinyi Zhang
"""

import pandas as pd
from tqdm import tqdm
import matplotlib.pyplot as plt
from qstock.data.money import stock_money


#正常显示画图时出现的中文和负号
from pylab import mpl
mpl.rcParams['font.sans-serif']=['SimHei']
mpl.rcParams['axes.unicode_minus']=False

#计算数据在系列时间周期内的收益率
def ret_date(data,w_list=[1,5,20,60,120]):
    data=data.fillna(method='ffill')
    df=pd.DataFrame()
    for w in w_list:
        df[str(w)+'日收益率%']=(((data/data.shift(w)-1)*100)
                            .round(2)
                            .T
                            .iloc[:,-1])
    return df

#计算某期间动量排名
def ret_rank(data,w_list=[1,5,20,60,120],c=4):
    #c为w_list里第几个
    rets=ret_date(data,w_list)
    col=rets.columns[c]
    rank_ret=rets.sort_values(col,ascending=False)
    return rank_ret

#同花顺概念动量排名
def ret_top(ths_rets,n=10):
    ths_top=pd.DataFrame()
    for c in ths_rets.columns:
        ths_top[c]=ths_rets.sort_values(c,ascending=False)[:n].index
    return ths_top

#获取同花顺概念指数动量排名名称列表
def ret_top_list(ths_top):
    alist=ths_top.values.tolist()
    words=' '.join([' '.join(s) for s in alist])
    word_list=words.split(' ')
    w_set=set(word_list)
    w_data=[]
    for w in w_set:
       w_data.append([w,word_list.count(w)/len(word_list)])
    return w_data

#运用在同花顺板块概念选股
def date_ret(data,w_list=[1,5,20,60,120]):
    data=data.fillna(method='ffill')
    df=pd.DataFrame()
    for w in w_list:
        df[str(w)+'日收益率%']=(((data/data.shift(w)-1)*100)
                            .round(2)
                            .T
                            .iloc[:,-1])
    return df

#计算某期间动量排名
def stock_rets_rank(data,w_list=[1,5,20,60,120],c=4):
    #c为w_list里第几个
    rets=date_ret(data,w_list)
    col=rets.columns[c]
    rank_ret=rets.sort_values(col,ascending=False)
    return rank_ret

#同花顺概念动量排名
def ths_top(ths_rets,n=10):
    ths_top=pd.DataFrame()
    for c in ths_rets.columns:
        ths_top[c]=ths_rets.sort_values(c,ascending=False)[:n].index
    return ths_top

#获取同花顺概念指数动量排名名称列表
def ths_top_list(ths_top):
    alist=ths_top.values.tolist()
    words=' '.join([' '.join(s) for s in alist])
    word_list=words.split(' ')
    w_set=set(word_list)
    w_data=[]
    for w in w_set:
       w_data.append([w,word_list.count(w)/len(word_list)])
    return w_data

#rps选股
class RPS(object):
    def __init__(self,data,w_list=[5,20,60,120,250]):
        self.data=data.fillna(method='ffill')
        self.w_list=w_list
        
    def cal_rps(self,ser):
        df=pd.DataFrame(ser.sort_values(ascending=False))
        df['n']=range(1,len(df)+1)
        df['rps']=(1-df['n']/len(df))*100
        return df.rps
    
    def all_rps(self,w):
        ret=(self.data/self.data.shift(w)-1).iloc[w:].fillna(0)
        rps=ret.T.apply(self.cal_rps)
        return rps.T

    def date_rps(self):
        print('抱歉，该接口仅限知识星球会员使用，详情可添加个人微信sky2blue2，获取8折优惠')
        pass
        
    def plot_stock_rps(self,stock,n=120):
        print('抱歉，该接口仅限知识星球会员使用，详情可添加个人微信sky2blue2，获取8折优惠')
        pass
        
#量价选股（参考欧奈尔的带柄茶杯形态）
#筛选价格和成交量突破N日阈值的个股
def find_price_vol_stock(data,n=120,rr=0.5):
    print('抱歉，该接口仅限知识星球会员使用，详情可添加个人微信sky2blue2，获取8折优惠')
    pass

#MM趋势选股（Mark Minervini’s Trend Template）
#参考文章https://zhuanlan.zhihu.com/p/165379657
#股票价格高于150天均线和200天均线
#1、150日均线高于200日均线
#2、200日均线上升至少1个月
#3、50日均线高于150日均线和200日均线
#4、股票价格高于50日均线
#5、股票价格比52周低点高30%
#6、股票价格在52周高点的25%以内
#7、相对强弱指数(RS)大于等于70，这里的相对强弱指的是股票与大盘对比，RS = 股票1年收益率 / 基准指数1年收益率
#这里将第七条RS改为欧奈尔的相对强弱指标，与RPS选股结合选择大于90值的股票

def MM_trend(close,sma=50,mma=150,lma=200):
    """close: 股票收盘价
    """
    print('抱歉，该接口仅限知识星球会员使用，详情可添加个人微信sky2blue2，获取8折优惠')
    pass

def tscode(code):
    return code+'.SH'if code.startswith('6') else code+'.SZ'

#资金流选股
def moneyflow_stock(codes,w_list=[3,5,10,20,60]):
    print('抱歉，该接口仅限知识星球会员使用，详情可添加个人微信sky2blue2，获取8折优惠')
    pass


#计算营收和利润增长得分
def cal_yoy(y,a):
    '''y是总资产报酬率时a=5
    y是营业收入增长率时a=10
    y是净资产收益率是a=15
    y是营业利润增长率a=20
    '''
    try:
        return 5+ min(round(y-a),5) if y>=a else 5+ max(round(y-a),-5)
    except:
        return 0
    
def cal_exp(y,a):
    '''y是毛利率或期间费用率a=0.5
     y是存货周转率a=2
     y是每股经营性现金流a=4
    '''
    try:
       return 5+min(round(y)/a,5) if y>0 else max(round(y)/a,-5)+5
    except:
       return 0
        
def cal_roa(y):
    #总资产报酬率打分
    try:
        return min(round((y-5)/0.5),10) if y>=5 else max(round(y-5),0)
    except:
        return 0
    
def cal_pepb(y,a,b):
    '''y是市净率时，a=3,b=0.4
      y是动态市盈率相对盈利增长率时，a=1,b=0.1
    '''
    try:
        return 5-max(round((y-a)/b),-5) if y<=a else 5-min(round((y-a)/b),5)
    except:
        return 0

#财务指标打分系统
##获取个股财务指标（指定某几个指标）+pe市盈率+pb市净率
fields='ts_code,ann_date,end_date,tr_yoy,op_yoy,\
         grossprofit_margin,expense_of_sales,inv_turn,eps,\
         ocfps,roe_yearly,roa2_yearly,netprofit_yoy,update_flag'
#update_flag财务数据是否更新过
def indicator_score(tudata,code,pepb=True,fields=fields):
    df=tudata.get_stock_indicator(stock=code,pepb=pepb,fields=fields)
    data=df.copy()
    '''(1)营业收入增长率打分'''
    data['营收得分']=data['tr_yoy'].apply(lambda y:cal_yoy(y,a=10))
    '''(2)营业利润增长率打分'''
    data['利润得分']=data['op_yoy'].apply(lambda y:cal_yoy(y,a=20))
    '''(3)毛利率打分'''
    #计算最近季度毛利率-前三季度平均毛利率
    data['gpm']=data['grossprofit_margin']-data['grossprofit_margin'].rolling(3).mean()
    data['毛利得分']=data['gpm'].apply(lambda y:cal_exp(y,a=0.5))
    '''(4)期间费用率打分'''
    #最近季度期间费用率-前三季度平均期间费用率
    data['exp']=data['expense_of_sales']-data['expense_of_sales'].rolling(3).mean()
    data['费用得分']=data['exp'].apply(lambda y:cal_exp(y,a=0.5))
    '''(5)周转率打分'''
    #（最近季度存货周转率-前三季度平均存货周转率）/前三季度平均存货周转率*100
    data['inv']=(data['inv_turn']-data['inv_turn'].rolling(3).mean())*100/data['inv_turn'].rolling(3).mean()
    data['周转得分']=data['inv'].apply(lambda y:cal_exp(y,a=2))
    '''(6)每股经营现金流打分'''
    #（最近三季度每股经营性现金流之和-最近三季度每股收益之和）/最近三季度每股收益之和*100
    data['ocf']=(data['ocfps'].rolling(3).sum()-data['eps'].rolling(3).sum())*100/data['eps'].rolling(3).sum()
    data['现金得分']=data['ocf'].apply(lambda y:cal_exp(y,a=4))
    '''(7)净资产收益率打分'''
    data['净资产得分']=data['roe_yearly'].apply(lambda y:cal_yoy(y,a=15))
    '''(8)总资产收益率打分'''
    data['总资产得分']=data['roa2_yearly'].apply(cal_roa)
    '''(9)市净率打分'''
    data['市净率得分']=data['pb'].apply(lambda y:cal_pepb(y,a=3,b=0.4))
    '''(10)动态市盈率相对盈利增长率打分'''
    #动态市盈率相对盈利增长率
    data['peg']=data['pe_ttm']/data['netprofit_yoy'].rolling(3).mean()
    data['市盈率得分']=data['peg'].apply(lambda y:cal_pepb(y,a=1,b=0.1))
    #计算总得分
    cols=['营收得分','利润得分','费用得分','周转得分','现金得分','净资产得分','总资产得分']
    data['总分']=data[cols].sum(axis=1)
    return data[cols+['总分']]

#计算所有股票财务指标总分
def all_indicator_score(tudata,name_code_dict):
    
    df=pd.DataFrame()
    for name,code in tqdm(name_code_dict.items()):
        try:
            d1=indicator_score(tudata,code)['总分']
            d1=pd.DataFrame(d1).rename(columns={'总分':name})
            df=pd.concat([df,d1],axis=1)
            
        except:
            continue
    return df
    

def indicator_score_rank(df,n1=10,n2=10):
    '''df:所有股票财务指标总分
    通过all_indicator_score(tudata,name_code_dict)获取
    '''
    result=df.T.sort_values(df.index[-1],ascending=False).iloc[:n1,-n1:]
    return result


# -*- coding: utf-8 -*-
"""
Created on Wed Oct  5 10:53:55 2022

@author: Jinyi Zhang
"""
import requests
from tqdm import tqdm
import pandas as pd

from bs4 import BeautifulSoup
from qstock.data.trade import latest_trade_date
from qstock.data.util import trans_num,ths_header

trade_date=latest_trade_date().replace('-','')
###################
#同花顺股票池
def ths_pool(ta=None):
    '''ta：输入技术形态选股
    可选："创月新高", "半年新高", "一年新高", "历史新高"，
     "创月新低", "半年新低", "一年新低", "历史新低"，'险资举牌'
     '连续上涨','连续下跌'，'持续放量'，'持续缩量'，'量价齐升'，
     '量价齐跌'，'强势股',f'u{n}',f'd{n}',n=
     10、20、30、60、90、250、500，突破n日均线
     如'u20'代表向上突破20日均线，'d10'：跌破10日均线
     
    '''
    u={"历史新高":1,"一年新高":2,"半年新高":3,"创月新高":4}
    d={"历史新低":1,"一年新低":2,"半年新低":3,"创月新低":4}
    ns=[10,20,30,60,90,250,500]
    uns=['u'+str(n) for n in ns]
    dns=['d'+str(n) for n in ns]
    uns_dict=dict(zip(uns,ns))
    dns_dict=dict(zip(dns,ns))
    
    if ta in ['ljqs','量价齐升']:
        return ths_price_vol('ljqs')
    elif ta in ['ljqd','量价齐跌']:
        return ths_price_vol('ljqd')
    elif ta in ['xzjp' ,'险资举牌']:
        return ths_xzjp()
    elif ta in ['lxsz','连续上涨']:
        return ths_up_down(flag='lxsz')
    elif ta in ['lxxd','连续下跌']:
        return ths_up_down(flag='lxxd')
    elif ta in ['cxfl','持续放量']:
        return ths_vol_change(flag='cxfl')
    elif ta in ['cxsl' ,'持续缩量']:
        return ths_vol_change(flag='cxsl')
   
    elif ta in u.keys():
        return ths_break_price(flag= "cxg",n=u[ta])
    elif ta in d.keys():
        return ths_break_price(flag= "cxd",n=d[ta])
    
    elif ta in uns:
        return ths_break_ma(flag='xstp',n=uns_dict[ta])
    elif ta in dns:
        return ths_break_ma(flag='xxtp',n=dns_dict[ta])
    else:
        return stock_strong_pool()

#获取东方财富网涨停（跌停）板股票池
def limit_pool(flag='u',date=None):
    '''date：日期如'20220916'
    flag='u'代表涨停板，'d'代表跌停，'s'代表强势股
    默认为最新交易日'''
    if date is None:
        date=trade_date
    if flag=='u' or flag=='up' or flag=='涨停':
        return stock_zt_pool(date)
    elif flag=='d' or flag=='down' or flag=='跌停':
        return stock_dt_pool(date)
    else:
        return stock_strong_pool(date)

#涨停股池

def stock_zt_pool(date= None) :
    """
    获取东方财富网涨停板行情
    date:日期
    """
    if date is None:
        date=trade_date
    url = 'http://push2ex.eastmoney.com/getTopicZTPool'
    params = {
        'ut': '7eea3edcaed734bea9cbfc24409ed989',
        'dpt': 'wz.ztzt',
        'Pageindex': '0',
        'pagesize': '10000',
        'sort': 'fbt:asc',
        'date': date,
        '_': '1621590489736',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    if data_json['data'] is None:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json['data']['pool'])
    temp_df.reset_index(inplace=True)
    temp_df['index'] = range(1, len(temp_df)+1)
    old_cols=['序号','代码','_','名称','最新价','涨跌幅','成交额(百万)','流通市值(百万)',
        '总市值(百万)', '换手率','连板数','首次封板时间','最后封板时间',
        '封板资金(百万)','炸板次数','所属行业','涨停统计',]
    temp_df.columns =  old_cols
    temp_df['涨停统计'] = (temp_df['涨停统计'].apply(lambda x: dict(x)['days']
                ).astype(str) + "/" + temp_df['涨停统计']
               .apply(lambda x: dict(x)['ct']).astype(str))
    new_cols=['代码','名称','涨跌幅','最新价','换手率','成交额(百万)','流通市值(百万)',
        '总市值(百万)','封板资金(百万)','首次封板时间','最后封板时间','炸板次数',
        '涨停统计','连板数','所属行业',]
    df = temp_df[new_cols].copy()
    df['首次封板时间'] = df['首次封板时间'].apply(lambda s:str(s)[-6:-4]+':'+str(s)[-4:-2])
    df['最后封板时间'] = df['最后封板时间'].apply(lambda s:str(s)[-6:-4]+':'+str(s)[-4:-2])
    df['最新价'] = df['最新价'] / 1000
   
    # 将object类型转为数值型
    ignore_cols = ['代码','名称','最新价','首次封板时间','最后封板时间','涨停统计','所属行业',]
    df = trans_num(df, ignore_cols)
    df[['成交额(百万)','流通市值(百万)','总市值(百万)','封板资金(百万)']]=(df[['成交额(百万)',
        '流通市值(百万)','总市值(百万)','封板资金(百万)']]/1000000)
    return df.round(3)

#跌停股池

def stock_dt_pool(date = None):
    """
    获取东方财富网跌停股池
    http://quote.eastmoney.com/ztb/detail#type=dtgc
    date: 交易日
    """
    if date is None:
        date=trade_date
    url = 'http://push2ex.eastmoney.com/getTopicDTPool'
    params = {
        'ut': '7eea3edcaed734bea9cbfc24409ed989',
        'dpt': 'wz.ztzt',
        'Pageindex': '0',
        'pagesize': '10000',
        'sort': 'fund:asc',
        'date': date,
        '_': '1621590489736',
    }
    res = requests.get(url, params=params)
    data_json = res.json()
    if data_json['data'] is None:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json['data']['pool'])
  
    old_cols=['代码','_','名称','最新价','涨跌幅','成交额(百万)','流通市值(百万)',
        '总市值(百万)','动态市盈率','换手率','封板资金(百万)','最后封板时间','板上成交额',
        '连续跌停','开板次数','所属行业',]
    temp_df.columns = old_cols
    new_cols=['代码','名称','涨跌幅','最新价','换手率','最后封板时间',
        '连续跌停','开板次数','所属行业','成交额(百万)','封板资金(百万)','流通市值(百万)',
        '总市值(百万)']
    df = temp_df[new_cols].copy()
    df['最新价'] = df['最新价'] / 1000
    df['最后封板时间'] = df['最后封板时间'].apply(lambda s:str(s)[-6:-4]+':'+str(s)[-4:-2])
    # 将object类型转为数值型
    ignore_cols = ['代码','名称','最新价','最后封板时间','所属行业',]
    df = trans_num(df, ignore_cols)
    df[['成交额(百万)','流通市值(百万)','总市值(百万)','封板资金(百万)']]=(df[['成交额(百万)',
        '流通市值(百万)','总市值(百万)','封板资金(百万)']]/1000000)
    return df.round(3)

def stock_strong_pool(date= None) :
    """
    获取东方财富网强势股池
    date：日期
    """
    if date is None:
        date=trade_date
    url = 'http://push2ex.eastmoney.com/getTopicQSPool'
    params = {
        'ut': '7eea3edcaed734bea9cbfc24409ed989',
        'dpt': 'wz.ztzt',
        'Pageindex': '0',
        'pagesize': '170',
        'sort': 'zdp:desc',
        'date': date,
        '_': '1621590489736',
    }
    res = requests.get(url, params=params)
    data_json = res.json()
    if data_json['data'] is None:
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json['data']['pool'])
   
    old_cols=['代码','_','名称','最新价','涨停价','_','涨跌幅',
        '成交额(百万)','流通市值(百万)','总市值(百万)', '换手率','是否新高','入选理由',
        '量比','涨速','涨停统计','所属行业',]
    temp_df.columns = old_cols
    temp_df['涨停统计'] = temp_df['涨停统计'].apply(lambda x: dict(x)['days']).astype(str) + "/" + temp_df['涨停统计'].apply(lambda x: dict(x)['ct']).astype(str)
    new_cols=['代码','名称','涨跌幅','最新价','涨停价','换手率','涨速','是否新高','量比',
              '涨停统计','入选理由','所属行业','成交额(百万)','流通市值(百万)','总市值(百万)', ]
    df = temp_df[new_cols].copy()
    df[['最新价','涨停价']] = df[['最新价','涨停价']] / 1000
    df[['成交额(百万)','流通市值(百万)','总市值(百万)']]=(df[['成交额(百万)',
        '流通市值(百万)','总市值(百万)']]/1000000)
    rr={1: '60日新高', 2: '近期多次涨停', 3: '60日新高且近期多次涨停'}
    df['入选理由']=df['入选理由'].apply(lambda s: rr[s])
    return df.round(2)

#############################################################################

url0='http://data.10jqka.com.cn/rank/'


def fetch_ths_data(url):
    res = requests.get(url, headers=ths_header())
    soup = BeautifulSoup(res.text, "lxml")
    try:
        total_page = soup.find(
            "span", attrs={"class": "page_info"}
        ).text.split("/")[1]
    except:
        total_page = 1
    df = pd.DataFrame()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        r = requests.get(url, headers=ths_header())
        temp_df = pd.read_html(r.text)[0]
        df = pd.concat([df, temp_df], ignore_index=True)
    return df


def ths_break_price(flag= "cxg",n=1):
    """
    获取同花顺技术选股-创新高个股
    flag: 'cxg'：创新高，'cxd'：创新低
    n=1,2,3,4,分别对应：历史新高（低）、一年新高（低）、半年新高（低）、创月新高（低）
    """
    page=1
    url = url0+f"{flag}/board/{n}/field/stockcode/order/asc/page/{page}/ajax/1/free/1/"
    df=fetch_ths_data(url)
        
    df.columns = ["序号","股票代码","股票简称","涨跌幅","换手率","最新价",
        "前期点位","前期点位日期",]
    
    df["股票代码"] = df["股票代码"].astype(str).str.zfill(6)
    df["涨跌幅"] = df["涨跌幅"].str.strip("%")
    df["换手率"] = df["换手率"].str.strip("%")
    df["前期点位日期"] = pd.to_datetime(df["前期点位日期"]).dt.date
    # 将object类型转为数值型
    ignore_cols = ["股票代码","股票简称","前期点位日期"]
    df = trans_num(df, ignore_cols)
    return df.iloc[:,1:]


def ths_up_down(flag='lxsz'):
    """
    同花顺技术选股连续上涨
    flag:'lxsz':连续上涨;'lxxd':连续下跌
    """
    page=1
    url = url0+f"{flag}/field/lxts/order/desc/page/{page}/ajax/1/free/1/"
    df=fetch_ths_data(url)
    
    df.columns = ["序号", "股票代码","股票简称","收盘价","最高价",
        "最低价","连涨天数","连续涨跌幅","累计换手率","所属行业",]
    df["股票代码"] = df["股票代码"].astype(str).str.zfill(6)
    df["连续涨跌幅"] = df["连续涨跌幅"].str.strip("%")
    df["累计换手率"] = df["累计换手率"].str.strip("%")
    # 将object类型转为数值型
    ignore_cols = ["股票代码","股票简称","所属行业"]
    df = trans_num(df, ignore_cols)
    return df.iloc[:,1:]

#同花顺技术选股持续放量（缩量）

def ths_vol_change(flag='cxfl'):
    """
    同花顺技术选股持续放量（缩量）
    flag='cxfl':持续放量;'cxsl':持续缩量
    """
    page=1
    url =url0+ f"{flag}/field/count/order/desc/ajax/1/free/1/page/{page}/free/1/"
    df=fetch_ths_data(url)
        
    df.columns = ["序号","股票代码","股票简称","涨跌幅","最新价","成交量",
        "基准日成交量","天数","阶段涨跌幅","所属行业",]
    df["股票代码"] = df["股票代码"].astype(str).str.zfill(6)
    df["涨跌幅"] = df["涨跌幅"].astype(str).str.strip("%")
    df["阶段涨跌幅"] = df["阶段涨跌幅"].astype(str).str.strip("%")
    ignore_cols = ["股票代码","股票简称","所属行业"]
    df = trans_num(df, ignore_cols)
    return df.iloc[:,1:]


def ths_break_ma(flag='xstp',n=20):
    """
    同花顺技术选股-向上（下）突破
    flag='xstp':向上突破；'xxtp':向下突破
    n:可选5、10、20、30、60、90、250、500，表示突破n日均线
    """
    page=1
    url =url0+ f"{flag}/board/{n}/order/asc/ajax/1/free/1/page/{page}/free/1/"
    df=fetch_ths_data(url)
        
    df.columns = ["序号","股票代码","股票简称","最新价","成交额","成交量(万)",
        "涨跌幅","换手率",]
    df["股票代码"] = df["股票代码"].astype(str).str.zfill(6)
    df[["涨跌幅","换手率"]] = df[["涨跌幅","换手率"]].apply(lambda s:s.astype(str).str.strip("%"))
    df['成交量(万)']=df['成交量(万)'].astype(str).str.strip("万")
    del df['成交额']
    ignore_cols = ["股票代码","股票简称"]
    df = trans_num(df, ignore_cols)
    return df.iloc[:,1:]

def ths_price_vol(flag='ljqs'):
    """
    返回同花顺技术选股-量价齐升（齐跌）
    flag: 'ljqs："量价齐升，'ljqd："量价齐跌
    """
    page=1
    url = url0+f"{flag}/field/count/order/desc/ajax/1/free/1/page/{page}/free/1/"
    df=fetch_ths_data(url)
        
    df.columns = ["序号","股票代码","股票简称","最新价",
        "天数","阶段涨幅","累计换手率","所属行业",]
    df["股票代码"] = df["股票代码"].astype(str).str.zfill(6)
    df["阶段涨幅"] = df["阶段涨幅"].astype(str).str.strip("%")
    df["累计换手率"] = df["累计换手率"].astype(str).str.strip("%")
    ignore_cols = ["股票代码","股票简称","所属行业",]
    df = trans_num(df, ignore_cols)
    return df.iloc[:,1:]

def ths_xzjp():
    """
    同花顺技术选股-险资举牌
    """
    url = "http://data.10jqka.com.cn/ajax/xzjp/field/DECLAREDATE/order/desc/ajax/1/free/1/"
    df=fetch_ths_data(url)
        
    df.columns = ["序号","举牌公告日","股票代码","股票简称","现价","涨跌幅",
        "举牌方","增持数量(万)","交易均价","增持数量占总股本比例","变动后持股总数(万)",
        "变动后持股比例","历史数据",]
    df["股票代码"] = df["股票代码"].astype(str).str.zfill(6)
    df["涨跌幅"] = df["涨跌幅"].astype(str).str.zfill(6)
    df["增持数量占总股本比例"] = df["增持数量占总股本比例"].astype(str).str.strip("%")
    df["增持数量(万)"] = df["增持数量(万)"].astype(str).str.strip("万").astype(float)
    df["变动后持股总数(万)"] = df["变动后持股总数(万)"].apply(lambda s: float(s.strip("亿"))*10000 
                                if s.endswith('亿') else float(s.strip("万")) )
    df["变动后持股比例"] = df["变动后持股比例"].astype(str).str.strip("%")
    
    df["举牌公告日"] = pd.to_datetime(df["举牌公告日"]).dt.date
    del df["历史数据"]
    ignore_cols = ["股票代码","股票简称","举牌方","举牌公告日"]
    df = trans_num(df, ignore_cols)
    return df.iloc[:,1:]



# -*- coding: utf-8 -*-
"""
Created on Sun Oct  9 09:03:15 2022

@author: Jinyi Zhang
"""
import pandas as pd  
import numpy as np
import matplotlib.pyplot as plt
from pylab import mpl
mpl.rcParams['font.sans-serif'] = ['SimHei']
mpl.rcParams['axes.unicode_minus'] = False
from datetime import timedelta
#关掉pandas的warnings
pd.options.mode.chained_assignment = None
from qstock.data.trade import get_data

#获取数据
def data_feed(code,index='hs300',start='20000101',end='',fqt=2):
    #获取个股数据
    df=get_data(code,start=start,end=end,fqt=fqt).fillna(method='ffill')
    #指数数据,作为参照指标
    df['index']=get_data(index,start,end).close.pct_change().fillna(0)
    #计算收益率
    df['rets']=df.close.pct_change().fillna(0)
    #计算买入持有和对比指数的累计收益率
    df['rets_line']=(df.rets+1.0).cumprod()
    df['index_line']=(df['index']+1.0).cumprod()
    return df

#有交易策略时
def trade_indicators(df):
    if 'capital_ret' not in df.columns:
        return
    # 计算资金曲线
    df['capital'] = (df['capital_ret'] + 1).cumprod()
    df=df.reset_index()
    # 记录买入或者加仓时的日期和初始资产
    df.loc[df['position'] > df['position'].shift(1), 'start_date'] = df['date']
    df.loc[df['position'] > df['position'].shift(1), 'start_capital'] = df['capital'].shift(1)
    df.loc[df['position'] > df['position'].shift(1), 'start_stock'] = df['close'].shift(1)
    # 记录卖出时的日期和当天的资产
    df.loc[df['position'] < df['position'].shift(1), 'end_date'] = df['date']
    df.loc[df['position'] < df['position'].shift(1), 'end_capital'] = df['capital']
    df.loc[df['position'] < df['position'].shift(1), 'end_stock'] = df['close']
    # 将买卖当天的信息合并成一个dataframe
    df_temp = df[df['start_date'].notnull() | df['end_date'].notnull()]

    df_temp['end_date'] = df_temp['end_date'].shift(-1)
    df_temp['end_capital'] = df_temp['end_capital'].shift(-1)
    df_temp['end_stock'] = df_temp['end_stock'].shift(-1)

    # 构建账户交易情况dataframe：'hold_time'持有天数，
    #'trade_return'该次交易盈亏,'stock_return'同期股票涨跌幅
    trade = df_temp.loc[df_temp['end_date'].notnull(), ['start_date', 'start_capital', 'start_stock',
                                                       'end_date', 'end_capital', 'end_stock']]
    trade['hold_time'] = (trade['end_date'] - trade['start_date']).dt.days
    trade['trade_return'] = trade['end_capital'] / trade['start_capital'] - 1
    trade['stock_return'] = trade['end_stock'] / trade['start_stock'] - 1

    trade_num = len(trade)  # 计算交易次数
    max_holdtime = trade['hold_time'].max()  # 计算最长持有天数
    average_change = trade['trade_return'].mean()  # 计算每次平均涨幅
    max_gain = trade['trade_return'].max()  # 计算单笔最大盈利
    max_loss = trade['trade_return'].min()  # 计算单笔最大亏损
    total_years = (trade['end_date'].iloc[-1] - trade['start_date'].iloc[0]).days / 365
    trade_per_year = trade_num / total_years  # 计算年均买卖次数

    # 计算连续盈利亏损的次数
    trade.loc[trade['trade_return'] > 0, 'gain'] = 1
    trade.loc[trade['trade_return'] < 0, 'gain'] = 0
    trade['gain'].fillna(method='ffill', inplace=True)
    # 根据gain这一列计算连续盈利亏损的次数
    rtn_list = list(trade['gain'])
    successive_gain_list = []
    num = 1
    for i in range(len(rtn_list)):
        if i == 0:
            successive_gain_list.append(num)
        else:
            if (rtn_list[i] == rtn_list[i - 1] == 1) or (rtn_list[i] == rtn_list[i - 1] == 0):
                num += 1
            else:
                num = 1
            successive_gain_list.append(num)
    # 将计算结果赋给新的一列'successive_gain'
    trade['successive_gain'] = successive_gain_list
    # 分别在盈利和亏损的两个dataframe里按照'successive_gain'的值排序并取最大值
    max_successive_gain = trade[trade['gain'] == 1].sort_values(by='successive_gain', \
                        ascending=False)['successive_gain'].iloc[0]
    max_successive_loss = trade[trade['gain'] == 0].sort_values(by='successive_gain', \
                        ascending=False)['successive_gain'].iloc[0]
    
    #  输出账户交易各项指标
    print ('\n==============每笔交易收益率及同期股票涨跌幅===============')
    print (trade[['start_date', 'end_date', 'trade_return', 'stock_return']])
    print ('\n====================账户交易的各项指标=====================')
    print ('交易次数为：%d   最长持有天数为：%d' % (trade_num, max_holdtime))
    print ('每次平均涨幅为：%f' % average_change)
    print ('单次最大盈利为：%f  单次最大亏损为：%f' % (max_gain, max_loss))
    print ('年均买卖次数为：%f' % trade_per_year)
    print ('最大连续盈利次数为：%d  最大连续亏损次数为：%d' % (max_successive_gain, max_successive_loss))
    return trade

def trade_performance(df,plot=True):
    if 'capital_ret' in df.columns:
        df1 = df.loc[:,['index','rets', 'capital_ret']]
        name_dict={'index':'基准指数','rets':'买入持有','capital_ret':'交易策略'}
    else:
        df1 = df.loc[:,['index','rets']]
        name_dict={'index':'基准指数','rets':'买入持有'}
    #df1.loc[df.index[0], ['index','rets']] = 0

    #计算收益率
    #累计收益率
    acc_ret=(df1+1).cumprod()
    #计算总收益率
    total_ret=acc_ret.iloc[-1]-1
    #年化收益率，假设一年250个交易日
    annual_ret=pow(1+total_ret,250/len(df1))-1
    #最大回撤
    md=((acc_ret.cummax()-acc_ret)/acc_ret.cummax()).max()
    exReturn=df1-0.03/250
    #计算夏普比率
    sharper_atio=np.sqrt(len(exReturn))*exReturn.mean()/df1.std()
    #计算CAPM里的alpha和beta系数
    beta0=df1[['rets','index']].cov().iat[0,1]/df['index'].var()
    alpha0=(annual_ret['rets']-annual_ret['index']*beta0)
    if 'capital_ret' in df1.columns:
        beta1=df1[['capital_ret','index']].cov().iat[0,1]/df['index'].var()
        alpha1=(annual_ret['capital_ret']-annual_ret['index']*beta1)
        alpha=[np.nan,alpha0,alpha1,]
        beta=[np.nan,beta0,beta1,]
    else:
        alpha=[np.nan,alpha0]
        beta=[np.nan,beta0]
    # 计算每一年(月,周)股票,资金曲线的收益
    year_ret = df1.resample('A').apply(lambda x: (x + 1.0).prod() - 1.0)
    month_ret = df1.resample('M').apply(lambda x: (x + 1.0).prod() - 1.0)
    week_ret = df1.resample('W').apply(lambda x: (x + 1.0).prod() - 1.0)
    #去掉缺失值
    year_ret.fillna(0,inplace=True)
    month_ret.fillna(0,inplace=True)
    week_ret.fillna(0,inplace=True)
    #计算胜率
    year_win_rate = year_ret.apply(lambda s:len(s[s>0])/len(s[s!=0]))
    month_win_rate =month_ret.apply(lambda s:len(s[s>0])/len(s[s!=0]))
    week_win_rate = week_ret.apply(lambda s:len(s[s>0])/len(s[s!=0]))
    result=pd.DataFrame()
    result['总收益率']=total_ret
    result['年化收益率']=annual_ret
    result['最大回撤']=md
    result['夏普比率']=sharper_atio
    result['Alpha']=alpha
    result['Beta']=beta
    result['年胜率']=year_win_rate
    result['月胜率']=month_win_rate
    result['周胜率']=week_win_rate
    result=result.T.rename(columns=name_dict)
    if plot:
        acc_ret=acc_ret.rename(columns=name_dict)
        acc_ret.plot(figsize=(15,7))
        plt.title('策略累计净值',size=15)
        plt.xlabel('')
        ax=plt.gca()
        ax.spines['right'].set_color('none')
        ax.spines['top'].set_color('none')
        plt.show()
    return result

def start_backtest(code,index='hs300',start='20000101',end='20220930',fqt=2,strategy=None):
    if not isinstance(code,str):
        trade_indicators(code.copy())
        return trade_performance(code.copy())
    print(f'回测标的：{code}')
    if end in ['',None]:
        end='至今'
    print(f'回测期间：{start}—{end}')
    d0=data_feed(code,index,start=start,end=end,fqt=fqt)
    if strategy is not None:
        df=strategy(d0)
        trade_indicators(df)
    else:
        df=d0
    return trade_performance(df)

def MR_Strategy(df,lookback=20,buy_threshold=-1.5,sell_threshold=1.5,cost=0.0):
    '''输入参数：
    df为数据表: 包含open,close,low,high,vol，标的收益率rets，指数收益率数据hs300
    lookback为均值回归策略参数，设置统计区间长度，默认20天
    buy_threshold:买入参数，均值向下偏离标准差的倍数，默认-1.5
    sell_threshold:卖出参数，均值向上偏离标准差的倍数，默认1.5
    cost为手续费+滑点价差，可以根据需要进行设置，默认为0.0
    '''
    #计算均值回归策略的score值
    ret_lb=df.rets.rolling(lookback).mean()
    std_lb=df.rets.rolling(lookback).std()
    df['score']=(df.rets-ret_lb)/std_lb
    df.fillna(0,inplace=True)
    #设计买卖信号，为尽量贴近实际，加入涨跌停不能买卖的限制
    #当score值小于-1.5且第二天开盘没有涨停发出买入信号设置为1
    df.loc[(df.score<buy_threshold) &(df['open'] < df['close'].shift(1) * 1.097), 'signal'] = 1
    #当score值大于1.5且第二天开盘没有跌停发出卖入信号设置为0
    df.loc[(df.score>sell_threshold) &(df['open'] > df['close'].shift(1) * 0.903), 'signal'] = 0
    df['position']=df['signal'].shift(1)
    df['position'].fillna(method='ffill',inplace=True)
    df['position'].fillna(0,inplace=True)
    #根据交易信号和仓位计算策略的每日收益率
    df.loc[df.index[0], 'capital_ret'] = 0
    #今天开盘新买入的position在今天的涨幅(扣除手续费)
    df.loc[df['position'] > df['position'].shift(1), 'capital_ret'] = \
                         (df['close'] / df['open']-1) * (1- cost) 
    #卖出同理
    df.loc[df['position'] < df['position'].shift(1), 'capital_ret'] = \
                   (df['open'] / df['close'].shift(1)-1) * (1-cost) 
    # 当仓位不变时,当天的capital是当天的change * position
    df.loc[df['position'] == df['position'].shift(1), 'capital_ret'] = \
                        df['rets'] * df['position']
    #计算策略累计收益率
    df['capital_line']=(df.capital_ret+1.0).cumprod()
    return df

def North_Strategy(data,window=252,stdev_n=1.5,cost=0.00):
    '''输入参数：
    data:包含北向资金数据
    window:移动窗口
    stdev_n:几倍标准差
    cost:手续费
    '''
    # 中轨
    df=data.copy().fillna(0)
    df['mid'] = df['北向资金'].rolling(window).mean()
    stdev = df['北向资金'].rolling(window).std()
    # 上下轨
    df['upper'] = df['mid'] + stdev_n * stdev
    df['lower'] = df['mid'] - stdev_n * stdev
    df['ret']=df.close/df.close.shift(1)-1
    df.fillna(0,inplace=True)
   
    #设计买卖信号
    #当日北向资金突破上轨线发出买入信号设置为1
    df.loc[df['北向资金']>df.upper, 'signal'] = 1
    #当日北向资金跌破下轨线发出卖出信号设置为0
    df.loc[df['北向资金']<df.lower, 'signal'] = 0
    df['position']=df['signal'].shift(1)
    df['position'].fillna(method='ffill',inplace=True)
    df['position'].fillna(0,inplace=True)
    #根据交易信号和仓位计算策略的每日收益率
    df.loc[df.index[0], 'capital_ret'] = 0
    #今天开盘新买入的position在今天的涨幅(扣除手续费)
    df.loc[df['position'] > df['position'].shift(1), 'capital_ret'] = \
                         (df.close/ df.open-1) * (1- cost) 
    #卖出同理
    df.loc[df['position'] < df['position'].shift(1), 'capital_ret'] = \
                   (df.open / df.close.shift(1)-1) * (1-cost) 
    # 当仓位不变时,当天的capital是当天的change * position
    df.loc[df['position'] == df['position'].shift(1), 'capital_ret'] = \
                        df['ret'] * df['position']
    #计算策略累计收益率
    df['capital_line']=(df.capital_ret+1.0).cumprod()
    return df

#海龟交易法则指数择时简单版回测
def TT_strategy(data,n1=20,n2=10):
    df=data.copy()
    #最近N1个交易日最高价
    df['H_N1']=df.high.rolling(n1).max()
    #最近N2个交易日最低价
    df['L_N2']=df.low.rolling(n2).max()
    #当日收盘价>昨天最近N1个交易日最高点时发出信号设置为1
    buy_index=df[df.close>df['H_N1'].shift(1)].index
    df.loc[buy_index,'signal']=1
    #将当日收盘价<昨天最近N2个交易日的最低点时收盘信号设置为0
    sell_index=df[df.close<df['L_N2'].shift(1)].index
    df.loc[sell_index,'signal']=0
    df['position']=df['signal'].shift(1)
    df['position'].fillna(method='ffill',inplace=True)
    d=df[df['position']==1].index[0]-timedelta(days=1)
    df1=df.loc[d:].copy()
    df1['position'][0]=0
    #当仓位为1时，买入持仓，当仓位为0时，空仓，计算资金净值
    df1['capital_ret']=df1.rets.values*df1['position'].values
    #计算策略累计收益率
    df1['capital_line']=(df1.capital_ret+1.0).cumprod()
    return df1










#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
@Author  ：Jinyi Zhang 
@Date    ：2022/9/29 20:20 
'''
#数据模块
#股票、债券、期货、基金等交易行情数据
from qstock.data.trade import *

#新闻数据
from qstock.data.news import *
#股票基本面数据
from qstock.data.fundamental import *
##行业、概念板块数据
from qstock.data.industry import *
#资金流向数据
from qstock.data.money import *
#宏观经济数据
from qstock.data.macro import *

#可视化模块
from qstock.plot.data_plot import *
from qstock.plot.chart_plot import *

#选股模块
from qstock.stock.stock_pool import *
from qstock.stock.ths_em_pool import *

#回测模块
from qstock.backtest.vec_backtest import *
from qstock.backtest.turtle import *
from qstock.backtest.bt_backtest import *

#问财数据
import pywencai

def wencai(query,query_type='stock'):
    '''获取结果'''
    result=pywencai.get(query=query,query_type=query_type)
    return result
    
from qstock import wencai


# -*- coding: utf-8 -*-
"""
Created on Fri Sep 30 13:38:58 2022

@author: Jinyi Zhang
"""
import re
import requests
import pandas as pd
import signal

import random 
import time

from functools import lru_cache
from tqdm import tqdm
from func_timeout import func_set_timeout
import multitasking
from bs4 import BeautifulSoup
from datetime import datetime
from functools import lru_cache
from qstock.data.util import ths_code_name,trans_num
from qstock.data.trade import latest_trade_date
from qstock.data import demjson
from qstock.stock.ths_em_pool import ths_header
signal.signal(signal.SIGINT, multitasking.killall)


#同花顺概念板块(简称ths)
#获取同花顺概念板块名称
def ths_index_name(flag='概念'):
    '''
    获取同花顺概念或行业板块名称
    flag='概念板块' or '行业板块'
    '''
    if flag=='概念板块' or flag=='概念':
        return ths_concept_name()
    else:
        names=list(ths_code_name.values())
    return names

#概念板块成分股
def ths_index_member(code=None):
    '''
    获取同花顺概念或行业板块成份股
    code:输入板块行业或概念代码或简称
    '''
    code_list=list(ths_code_name)+list(ths_code_name.values())
    if code in code_list:
        return ths_industry_member(code)
    else:
        return ths_concept_member(code)


#多线程获取同花顺指数数据
def ths_index_price(flag='概念'):
    df_list=[]
    codes=ths_index_name(flag)
    pbar = tqdm(total=len(codes),leave=False)
    
    #@multitasking.task
    #@func_set_timeout(5)
    def run(code):
        try: 
            temp=ths_index_data(code)
            temp[code]=temp.close
            df_list.append(temp[code])
            pbar.update()
        except:
            pass
    
    
    for code in codes:
        try:
            run(code)
        except:
            continue
    #multitasking.wait_for_tasks()
    # 转换为dataframe
    df = pd.concat(df_list, axis=1)
    return df



#概念指数行情数据
def ths_index_data(code=None):
    '''
    获取同花顺概念或行业板块指数行情数据
    code:输入板块行业或概念代码或简称
    '''
    code_list=list(ths_code_name)+list(ths_code_name.values())
    if code in code_list:
        return ths_industry_data(code)
    else:
        return ths_concept_data(code)

def ths_name_code(code):
    """
    获取同花顺行业对应代码
    """
    if code.isdigit():
        return code
    name_code={value:key for key, value in ths_code_name.items()}
    code=name_code[code]
    return code

#修正函数，但只能获取第一页的数据，无法获取翻页的
def ths_industry_member(code= "半导体"):
    """
    同花顺-板块-行业板块
    http://q.10jqka.com.cn/gn/detail/code/881270/
   
    """
    if code.isdigit():
        symbol=code
    else:
        symbol=ths_name_code(code)
    # 定义要请求的网页 URL 
    url = f"http://q.10jqka.com.cn/thshy/detail/code/{symbol}/"
    
    # 设置请求头，模拟浏览器访问 
    try: 
       # 发送 HTTP 请求获取网页内容 
       response = requests.get(url,  headers=ths_header()) 
       # 检查响应状态码 
       response.raise_for_status()  
 
       # 设置响应内容的编码 
       response.encoding  = response.apparent_encoding  
       # 获取网页源代码 
       html = response.text  
 
       # 使用 BeautifulSoup 解析 HTML 
       soup = BeautifulSoup(html, 'html.parser')  
 
       # 查找表格元素，这里假设表格的 class 为 'm-table'，你可以根据实际情况修改 
       table = soup.find('table',  class_='m-table') 
 
       if table: 
           # 使用 pandas 读取表格数据 
           df = pd.read_html(str(table))[0] 
           return df           
       else: 
           print('未找到表格元素') 
    except requests.RequestException as e: 
       print(f'请求出错: {e}') 
    except Exception as e: 
       print(f'发生其他错误: {e}') 

def ths_industry_data(code= "半导体及元件",start= "20200101",end= None) :
    """
    获取同花顺行业板块指数数据
    http://q.10jqka.com.cn/gn/detail/code/301558/
    start: 开始时间
    end: 结束时间
    """
    if end is None:
        end=latest_trade_date()
    if code.isdigit():
        symbol=code
    else:
        symbol=ths_name_code(code)
    df = pd.DataFrame()
    current_year = datetime.now().year
    for year in range(2000, current_year + 1):
        url = f"http://d.10jqka.com.cn/v4/line/bk_{symbol}/01/{year}.js"
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Referer": "http://q.10jqka.com.cn",
            "Host": "d.10jqka.com.cn",
        }
        res = requests.get(url, headers=headers)
        data_text = res.text
        try:
            demjson.decode(data_text[data_text.find("{") : -1])
        except:
            continue
        temp_df = demjson.decode(data_text[data_text.find("{") : -1])
        temp_df = pd.DataFrame(temp_df["data"].split(";"))
        temp_df = temp_df.iloc[:, 0].str.split(",", expand=True)
        df = pd.concat([df, temp_df], ignore_index=True)

    if len(df.columns) == 11:
        df.columns = ["日期","开盘价","最高价","最低价","收盘价","成交量",
            "成交额","_","_","_","_",]
    else:
        df.columns = ["日期","开盘价","最高价","最低价","收盘价","成交量",
            "成交额","_", "_","_","_","_",]
    
    df["日期"] = pd.to_datetime(df["日期"]).dt.date
    c1 = pd.to_datetime(start) < df["日期"]
    c2 = pd.to_datetime(end) > df["日期"]
    df = df[c1 & c2]
    cols1 = ["日期","开盘价","最高价","最低价","收盘价","成交量"]
    cols2 = ['date', 'open', 'high', 'low', 'close', 'volume']
    df = df.rename(columns=dict(zip(cols1,cols2)))[cols2]
    df.set_index('date',inplace=True)
    ignore_cols = ['date']
    df = trans_num(df, ignore_cols)
    return df

@lru_cache()
def ths_concept_name_code():
    """
    同花顺概念板块概念名称
    http://q.10jqka.com.cn/gn/detail/code/301558/
    """
    page=1
    url = f"http://q.10jqka.com.cn/gn/index/field/addtime/order/desc/page/{page}/ajax/1/"
    
    res = requests.get(url, headers=ths_header())
    soup = BeautifulSoup(res.text, "lxml")
    total_page = soup.find("span", attrs={"class": "page_info"}).text.split("/")[1]
    df = pd.DataFrame()
    for page in tqdm(range(1, int(total_page) + 1), leave=False):
        r = requests.get(url, headers=ths_header())
        soup = BeautifulSoup(r.text, "lxml")
        url_list = []
        for item in (
            soup.find("table", attrs={"class": "m-table m-pager-table"})
            .find("tbody")
            .find_all("tr")):
            inner_url = item.find_all("td")[1].find("a")["href"]
            url_list.append(inner_url)
        temp_df = pd.read_html(r.text)[0]
        temp_df["网址"] = url_list
        df = pd.concat([df, temp_df], ignore_index=True)
    df = df[["日期", "概念名称", "成分股数量", "网址"]]
    df["日期"] = pd.to_datetime(df["日期"]).dt.date
    df["成分股数量"] = pd.to_numeric(df["成分股数量"])
    df["代码"] = df["网址"].str.split("/", expand=True).iloc[:, 6]
    df.drop_duplicates(keep="last", inplace=True)
    df.reset_index(inplace=True, drop=True)

    # 处理遗漏的板块
    url = "http://q.10jqka.com.cn/gn/detail/code/301558/"
    r = requests.get(url, headers=ths_header())
    soup = BeautifulSoup(r.text, "lxml")
    need_list = [
        item.find_all("a")
        for item in soup.find_all(attrs={"class": "cate_group"})
    ]
    temp_list = []
    for item in need_list:
        temp_list.extend(item)
    temp_df = pd.DataFrame(
        [
            [item.text for item in temp_list],
            [item["href"] for item in temp_list],
        ]
    ).T
    temp_df.columns = ["概念名称", "网址"]
    temp_df["日期"] = None
    temp_df["成分股数量"] = None
    temp_df["代码"] = (
        temp_df["网址"].str.split("/", expand=True).iloc[:, 6].tolist()
    )
    temp_df = temp_df[["日期", "概念名称", "成分股数量", "网址", "代码"]]
    df = pd.concat([df, temp_df], ignore_index=True)
    df.drop_duplicates(subset=["概念名称"], keep="first", inplace=True)
    return df

def ths_concept_name():
    """
    获取同花顺概念板块-概念名称
    """
    ths_df = ths_concept_name_code()
    name_list = ths_df["概念名称"].tolist()
    return name_list

def ths_concept_code():
    """
    获取同花顺概念板块-概念代码
    """
    ths_df = ths_concept_name_code()
    name_list = ths_df["概念名称"].tolist()
    code_list = list(ths_df['代码'])
    name_code_dict = dict(zip(name_list, code_list))
    return name_code_dict

#修正函数，但只能获取第一页的数据，无法获取翻页的
def ths_concept_member(code= "阿里巴巴概念"):
    """
    同花顺-板块-概念板块
    https://q.10jqka.com.cn/gn/detail/code/301558/
   
    """
    if code.isdigit():
        symbol=code
    else:
        symbol=ths_concept_code()[code]
    # 定义要请求的网页 URL 
    url = f" https://q.10jqka.com.cn/gn/detail/code/{symbol}/"
    
    # 设置请求头，模拟浏览器访问 
    try: 
       # 发送 HTTP 请求获取网页内容 
       response = requests.get(url,  headers=ths_header()) 
       # 检查响应状态码 
       response.raise_for_status()  
 
       # 设置响应内容的编码 
       response.encoding  = response.apparent_encoding  
       # 获取网页源代码 
       html = response.text  
 
       # 使用 BeautifulSoup 解析 HTML 
       soup = BeautifulSoup(html, 'html.parser')  
 
       # 查找表格元素，这里假设表格的 class 为 'm-table'，你可以根据实际情况修改 
       table = soup.find('table',  class_='m-table') 
 
       if table: 
           # 使用 pandas 读取表格数据 
           df = pd.read_html(str(table))[0] 
           return df           
       else: 
           print('未找到表格元素') 
    except requests.RequestException as e: 
       print(f'请求出错: {e}') 
    except Exception as e: 
       print(f'发生其他错误: {e}') 

def ths_concept_data(code='白酒概念',start= "2020"):
    """
    同花顺-板块-概念板块-指数数据
    http://q.10jqka.com.cn/gn/detail/code/301558/
    start: 开始年份; e.g., 2019
    """
    code_map = ths_concept_code()
    symbol_url = f"http://q.10jqka.com.cn/gn/detail/code/{code_map[code]}/"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
    }
    r = requests.get(symbol_url, headers=headers)
    soup = BeautifulSoup(r.text, "lxml")
    symbol_code = (
        soup.find("div", attrs={"class": "board-hq"}).find("span").text
    )
    df = pd.DataFrame()
    current_year = datetime.now().year
    for year in range(int(start), current_year + 1):
        url = f"http://d.10jqka.com.cn/v4/line/bk_{symbol_code}/01/{year}.js"
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36",
            "Referer": "http://q.10jqka.com.cn",
            "Host": "d.10jqka.com.cn",
        }
        r = requests.get(url, headers=headers)
        data_text = r.text
        try:
            demjson.decode(data_text[data_text.find("{") : -1])
        except:
            continue
        temp_df = demjson.decode(data_text[data_text.find("{") : -1])
        temp_df = pd.DataFrame(temp_df["data"].split(";"))
        temp_df = temp_df.iloc[:, 0].str.split(",", expand=True)
        df = pd.concat([df, temp_df], ignore_index=True)
    if df.columns.shape[0] == 12:
        df.columns = ["日期","开盘价","最高价","最低价","收盘价","成交量",
            "成交额","_","_","_","_","_",]
    else:
        df.columns = ["日期","开盘价","最高价","最低价","收盘价", "成交量",
            "成交额","_","_","_","_",]
    df = df[["日期","开盘价","最高价","最低价","收盘价","成交量","成交额",]]
    cols1 = ["日期","开盘价","最高价","最低价","收盘价","成交量"]
    cols2 = ['date', 'open', 'high', 'low', 'close', 'volume']
    df = df.rename(columns=dict(zip(cols1,cols2)))[cols2]
    df.set_index('date',inplace=True)
    ignore_cols = ['date']
    df = trans_num(df, ignore_cols)
    return df
    
# 辅助函数：获取板块代码及类型（行业/概念）
def __get_sector_code(name: str) -> tuple:
    """
    根据板块名称或代码获取对应的板块代码和类型（行业/概念）
    :param name: 板块名称或代码
    :return: (板块代码, 类型)
    """
    # 检查是否为板块代码
    if re.match(r"^BK\d+", name):
        # 查询行业板块
        industry_df = em_index_name('industry')
        if name in industry_df['板块代码'].values:
            return (name, 'industry')
        # 查询概念板块
        concept_df = em_index_name('concept')
        if name in concept_df['板块代码'].values:
            return (name, 'concept')
        raise ValueError(f"无效的板块代码: {name}")
    else:
        # 按名称查询
        industry_df = em_index_name('industry')
        if name in industry_df['板块名称'].values:
            code = industry_df.loc[industry_df['板块名称'] == name, '板块代码'].iloc[0]
            return (code, 'industry')
        concept_df = em_index_name('concept')
        if name in concept_df['板块名称'].values:
            code = concept_df.loc[concept_df['板块名称'] == name, '板块代码'].iloc[0]
            return (code, 'concept')
        raise ValueError(f"无效的板块名称: {name}")

###########获取东方财富网（简称em)行业概念板块数据
# 板块名称列表（带缓存）
@lru_cache()
def em_index_name(sector_type: str = 'concept') -> pd.DataFrame:
    """
    获取行业或概念板块名称列表
    :param sector_type: 板块类型，'concept'（概念）或'industry'（行业）
    :return: DataFrame
    """
    # 请求参数配置
    config = {
        'concept': {
            'url': "https://79.push2.eastmoney.com/api/qt/clist/get",
            'fs': "m:90 t:3 f:!50",
            'fields': "f2,f3,f4,f8,f12,f14,f15,f16,f17,f18,f20,f21,f24,f25,f22,f33,f11,f62,f128,f124,f107,f104,f105,f136"
        },
        'industry': {
            'url': "https://17.push2.eastmoney.com/api/qt/clist/get",
            'fs': "m:90 t:2 f:!50",
            'fields': "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,f26,f22,f33,f11,f62,f128,f136,f115,f152,f124,f107,f104,f105,f140,f141,f207,f208,f209,f222"
        }
    }
    sector_type_dict={'1':'concept','2':'industry','c':'concept','i':'industry','概念':'concept','行业':'industry',
                      '行业板块':'concept','概念板块':'industry','concept':'concept','industry':'industry'}
    sector_type=sector_type_dict[str(sector_type)]
    cfg = config.get(sector_type)
    if not cfg:
        raise ValueError("sector_type 必须为 'concept' 或 'industry'")

    params = {
        "pn": "1", "pz": "50000", "po": "1", "np": "2",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2", "invt": "2", "fid": "f3",
        "fs": cfg['fs'], "fields": cfg['fields'],
        "_": "1626075887768"
    }
    
    response = requests.get(cfg['url'], params=params)
    data = response.json()
    df = pd.DataFrame(data['data']['diff']).T
    df.reset_index(inplace=True)

    # 列名处理
    if sector_type == 'concept':
        df.columns = [
            "排名", "最新价", "涨跌幅", "涨跌额", "换手率", "_", "板块代码", "板块名称",
            "_", "_", "_", "_", "总市值", "_", "_", "_", "_", "_", "_",
            "上涨家数", "下跌家数", "_", "_", "领涨股票", "_", "_", "领涨股票-涨跌幅"
        ]
    else:
        df.columns = [
            "排名", "-", "最新价", "涨跌幅", "涨跌额", "-", "_", "-", "换手率", "-",
            "-", "-", "板块代码", "-", "板块名称", "-", "-", "-", "-", "总市值",
            "-", "-", "-", "-", "-", "-", "-", "-", "上涨家数", "下跌家数",
            "-", "-", "-", "领涨股票", "-", "-", "领涨股票-涨跌幅", "-", "-", "-", "-", "-"
        ]
    
    # 统一列选择及类型转换
    cols = [
        "排名", "板块名称", "板块代码", "最新价", "涨跌额", "涨跌幅",
        "总市值", "换手率", "上涨家数", "下跌家数", "领涨股票", "领涨股票-涨跌幅"
    ]
    df = df[cols]
    numeric_cols = ["最新价", "涨跌额", "涨跌幅", "总市值", "换手率", 
                   "上涨家数", "下跌家数", "领涨股票-涨跌幅"]
    df[numeric_cols] = df[numeric_cols].apply(pd.to_numeric, errors='coerce')
    return df

# 板块成分股
def em_index_member(name: str) -> pd.DataFrame:
    """
    获取板块成分股
    :param name: 板块名称或代码
    :return: 成分股数据
    """
    code, _ = __get_sector_code(name)
    url = "https://29.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1", "pz": "50000", "po": "1", "np": "2",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2", "invt": "2", "fid": "f3",
        "fs": f"b:{code} f:!50",
        "fields": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f12,f13,f14,f15,f16,f17,f18,f20,f21,f23,f24,f25,f22,f11,f62,f128,f136,f115,f152,f45",
        "_": "1626081702127"
    }
    response = requests.get(url, params=params)
    data = response.json()
    df = pd.DataFrame(data['data']['diff']).T
    df.reset_index(inplace=True)
    df.columns = [
        "序号", "_", "最新价", "涨跌幅", "涨跌额", "成交量", "成交额", "振幅",
        "换手率", "市盈率-动态", "_", "_", "代码", "_", "名称", "最高", "最低",
        "今开", "昨收", "_", "_", "_", "市净率", "_", "_", "_", "_", "_", "_", "_", "_", "_", "_"
    ]
    df = df[[
        "序号", "代码", "名称", "最新价", "涨跌幅", "涨跌额", "成交量", "成交额",
        "振幅", "最高", "最低", "今开", "昨收", "换手率", "市盈率-动态", "市净率"
    ]]
    numeric_cols = ["最新价", "涨跌幅", "涨跌额", "成交量", "成交额", "振幅",
                    "最高", "最低", "今开", "昨收", "换手率", "市盈率-动态", "市净率"]
    df[numeric_cols] = df[numeric_cols].apply(pd.to_numeric, errors='coerce')
    return df

# 实时行情
def em_index_realtime(name: str) -> pd.DataFrame:
    """
    获取板块实时行情
    :param name: 板块名称或代码
    :return: 实时行情数据
    """
    code, _ = __get_sector_code(name)
    url = "https://91.push2.eastmoney.com/api/qt/stock/get"
    fields = {
        "f43": "最新价", "f44": "最高", "f45": "最低", "f46": "开盘",
        "f47": "成交量", "f48": "成交额", "f170": "涨跌幅", 
        "f171": "振幅", "f168": "换手率", "f169": "涨跌额"
    }
    params = {
        "fields": ",".join(fields.keys()),
        "mpi": "1000", "invt": "2", "fltt": "1",
        "secid": f"90.{code}", 
        "ut": "fa5fd1943c7b386f172d6893dbfba10b"
    }
    response = requests.get(url, params=params)
    data = response.json()['data']
    df = pd.Series(data).reset_index()
    df.columns = ['字段', '值']
    df['字段'] = df['字段'].map(fields)
    # 单位转换
    df['值'] = pd.to_numeric(df['值'], errors='coerce') / 100
    df.loc[df['字段'].isin(['成交量', '成交额']), '值'] *= 100
    return df

# 历史行情数据
def em_index_data(name: str, freq: str = 'd', 
               start: str = None, end: str = None,
               fqt: str = '') -> pd.DataFrame:
    """
    获取板块历史行情
    :param name: 板块名称或代码
    :param freq: 周期（d/w/m）
    :param start: 起始日期（YYYYMMDD）
    :param end: 结束日期（YYYYMMDD）
    :param fqt: 复权类型（''/qfq/hfq）
    :return: 历史行情数据
    """
    code, sector_type = __get_sector_code(name)
    period_map = {'d': '101', 'w': '102', 'm': '103'}
    adjust_map = {'': '0', 'qfq': '1', 'hfq': '2', '1': '1', '2': '2'}
    
    # URL选择
    if sector_type == 'industry':
        url = "http://7.push2his.eastmoney.com/api/qt/stock/kline/get"
    else:
        url = "https://91.push2his.eastmoney.com/api/qt/stock/kline/get"
    
    params = {
        "secid": f"90.{code}",
        "ut": "fa5fd1943c7b386f172d6893dbfba10b",
        "fields1": "f1,f2,f3,f4,f5,f6",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
        "klt": period_map[freq],
        "fqt": adjust_map[fqt],
        "beg": start or "19000101",
        "end": end or "20500101",
        "lmt": "1000000",
        "_": str(int(pd.Timestamp.now().timestamp()*1000))
    }
    
    response = requests.get(url, params=params)
    data = response.json()
    rows = [item.split(',') for item in data['data']['klines']]
    df = pd.DataFrame(rows, columns=[
        '日期', '开盘', '收盘', '最高', '最低', '成交量', '成交额',
        '振幅', '涨跌幅', '涨跌额', '换手率'
    ])[['日期', '开盘', '收盘', '最高', '最低', '涨跌幅', 
        '涨跌额', '成交量', '成交额', '振幅', '换手率']]
    numeric_cols = df.columns[1:]
    df[numeric_cols] = df[numeric_cols].apply(pd.to_numeric, errors='coerce')
    return df


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
'''
@Author  ：Jinyi Zhang 
@Date    ：2022/9/29 20:22 
'''

from .chart_plot import (HA_kline,kline,stock_line,chart_line,chart_bar,
                         chart_inv_bar,chart_pie,chart_heatmap,chart_heatmap_color,
                         chart_map,cut_word,cloud_data,chart_wordcloud,
                         chart_calendar,chart_box,)


from .data_plot import (line,scatter,pie,bar,box,violin,hist,hist_kde,treemap,
                       plot_north_money, plot_stock_money,plot_ichimoku)


# -*- coding: utf-8 -*-
"""
Created on Fri May  5 19:16:32 2023

@author: tkfy9
"""
import re
import pandas as pd
import requests
import random
import os
import json
import execjs
import datetime as dt

WENCAI_LOGIN_URL = {
    "scrape_transaction": 'http://www.iwencai.com/traceback/strategy/transaction',
    "scrape_report": 'http://www.iwencai.com/traceback/strategy/submit',
    'strategy': 'http://www.iwencai.com/traceback/strategy/submit',
    "search": "http://www.iwencai.com/data-robot/extract-new",
    'recommend_strategy': 'http://www.iwencai.com/traceback/list/get-strategy',
    'backtest': 'http://backtest.10jqka.com.cn/backtest/app.html#/backtest',
    'lastjs':'http://x.10jqka.com.cn/stockpick?tid=stockpick&ts=1&qs=pc_~soniu~stock~stock~znxg~topbar&allow_redirect=false&zhineng=opened'
}

WENCAI_CRAWLER_URL = {
    'history_detail': 'http://backtest.10jqka.com.cn/backtestonce/historydetail?sort_by=desc&id={backtest_id}&start_date={start_date}&end_date={end_date}&period={period}',
    "backtest": "http://backtest.10jqka.com.cn/backtestonce/backtest",
    "yieldbacktest": "http://backtest.10jqka.com.cn/tradebacktest/yieldbacktest",
    "history_pick": 'http://backtest.10jqka.com.cn/tradebacktest/historypick?query={query}&hold_num={hold_num}&trade_date={trade_date}',
    'eventbacktest': 'http://backtest.10jqka.com.cn/eventbacktest/backtest',
    'lastjs':'http://d.10jqka.com.cn/v2/time/{}/last.js',
    'search': 'http://x.10jqka.com.cn/unifiedwap/unified-wap/v2/result/get-robot-data'

}

WENCAI_HEADERS = {
    'backtest': {
        'Host': "backtest.10jqka.com.cn",
        'Origin': "http://backtest.10jqka.com.cn",
        "Referer": "http://backtest.10jqka.com.cn/backtest/app.html",
    },
    'lastjs':{
        'Host': "d.10jqka.com.cn",
        # 'Origin': "http://backtest.10jqka.com.cn",
        "Referer": "http://x.10jqka.com.cn/",
        "User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.93 Safari/537.36"
    }

}


class WencaiCookie:

    def __init__(self):
        self.json_path = os.path.dirname(__file__) + '/cookies.json'

    def getHeXinVByHttp(self):
        with open(os.path.dirname(os.path.dirname(__file__)) + '\\data\\hexin.js', 'r') as f:
            jscontent = f.read()
        context = execjs.compile(jscontent)
        return context.call("v")

    def setHexinByJson(self, source, cookies=None):
        if cookies is None: cookies = dict()
        henxin_v = self.getHeXinVByHttp()
        cookies[source] = henxin_v
        cookies['expire_time'] = dt.datetime.today().strftime('%Y-%m-%d %H:%M:%S')
        with open(self.json_path, 'w') as f:
            json.dump(cookies, f)
        return henxin_v

    def is_expire(self, expire_time, days=3):
        delta = dt.datetime.today() - dt.datetime.strptime(expire_time, '%Y-%m-%d %H:%M:%S')
        if delta.days >= days:
            return True
        else:
            return False

    def getHexinVByJson(self, source):
        json_path = os.path.dirname(__file__) + '/cookies.json'
        if os.path.exists(json_path):
            with open(json_path, 'r') as f:
                cookies = json.load(f)

            if source in cookies:
                if not self.is_expire(cookies['expire_time']):
                    return cookies[source]
                else:
                    return self.setHexinByJson(source=source, cookies=cookies)
            else:
                return self.setHexinByJson(source=source, cookies=cookies)
        else:
            self.setHexinByJson(source=source)


class Session(requests.Session):
    headers = {
        "Accept": "application/json,text/javascript,*/*;q=0.01",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.8",
        'Connection': 'keep-alive',
        'Content-Type': "application/x-www-form-urlencoded; charset=UTF-8",
        'User-Agent': "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36",
        'X-Requested-With': "XMLHttpRequest"

    }

    def __init__(self, proxies=None, verify=False):
        requests.Session.__init__(self)
        self.headers.update(Session.headers)
        if proxies is not None:
            if not isinstance(proxies, (list, dict)):
                raise TypeError('proxies should be list or dict')
            if isinstance(proxies, list):
                proxies = random.choice(proxies)
        self.proxies = proxies
        self.verify = verify

    def update_headers(self, source, add_headers, force_cookies=False):
        if force_cookies:
            self.headers['hexin-v'] = WencaiCookie().getHeXinVByHttp()
        else:
            self.headers['hexin-v'] = WencaiCookie().getHexinVByJson(source=source)
        if add_headers is not None:
            if not isinstance(add_headers, dict):
                raise TypeError('update_headers should be `dict` type.')
            for k, v in add_headers.items():
                self.headers[k] = v

    def get_result(self, url, source=None, force_cookies=False, add_headers=None, **kwargs):
        self.update_headers(add_headers=add_headers, source=source, force_cookies=force_cookies)
        if self.proxies is None:
            return super(Session, self).get(url=url, **kwargs)
        else:
            return super(Session, self).get(url=url, proxies=self.proxies, verify=self.verify, **kwargs)

    def post_result(self, url, source=None, data=None, json=None, add_headers=None, force_cookies=False, **kwargs):
        self.update_headers(add_headers=add_headers, source=source, force_cookies=force_cookies)
        if self.proxies is None:
            return super(Session, self).post(url=url, data=data, json=json, **kwargs)
        else:
            return super(Session, self).post(url=url, data=data, json=json, proxies=self.proxies, verify=self.verify,
                                             **kwargs)

cookies = WencaiCookie()
session = Session(proxies=None, verify=False)
session.headers.update({'Host':'www.iwencai.com'})

def wencai(question):
    '''
    question:输入你要问的条件，不同条件使用“，”或“；”或空格
    如'均线多头排列'
    
    '''
    payload = {
            "question": question,
            "page": 100,
            "perpage": 100,
            "log_info": '{"input_type": "typewrite"}',
            "source": "Ths_iwencai_Xuangu",
            "version": 2.0,
            "secondary_intent": "",
            "query_area": "",
            "block_list": "",
            "add_info": '{"urp": {"scene": 1, "company": 1, "business": 1}, "contentType": "json", "searchInfo": true}'
        }

    r = session.post_result(url=WENCAI_CRAWLER_URL['search'],
                                     data=payload, force_cookies=True)
    try:
        result = r.json()['data']['answer'][0]['txt'][0]['content']['components'][0]['data']['datas']
    except:
        result={}
        print('没有你要的结果')

    def _re_str(x: str):
        _re = re.findall('(.*):前复权', x)
        if len(_re) >= 1:
            x = _re[-1]
        check_date = re.search(r"(\d{4}\d{1,2}\d{1,2})",x)
        if check_date is not None:
            return x.replace('[{}]'.format(check_date.group()), '')
        else:
            return x

    data = pd.DataFrame().from_dict(result)
    if not data.empty:
        columns = {i: _re_str(i) for i in data.columns}
        data = data.rename(columns=columns)
        for col in ['market_code', 'code', '关键词资讯', '涨跌幅']:
            if col in data.columns:
                del data[col]
    return data